@@ -1,12 +1,12 @@
 """Classes related to card endpoints
 """
+from __future__ import annotations
 
 from datetime import datetime
-from typing import ClassVar, Optional
+from typing import Any, ClassVar, Optional
 from uuid import UUID
 
 from pydantic.fields import Field
-from typing_extensions import Self
 
 from metabase_tools.exceptions import (
     EmptyDataReceived,
@@ -15,11 +15,11 @@ from metabase_tools.exceptions import (
 )
 from metabase_tools.metabase import MetabaseApi
 from metabase_tools.models.collection import Collection
-from metabase_tools.models.generic import GenericTemplateWithArchive
+from metabase_tools.models.generic import GA, GenericWithArchive
 from metabase_tools.models.user import User
 
 
-class Card(GenericTemplateWithArchive):
+class Card(GenericWithArchive):
     """Card object class with related methods"""
 
     BASE_EP: ClassVar[str] = "/card"
@@ -28,7 +28,7 @@ class Card(GenericTemplateWithArchive):
     archived: bool
     collection_position: Optional[int]
     table_id: Optional[int]
-    result_metadata: Optional[list[dict]]
+    result_metadata: Optional[list[dict[str, Any]]]
     creator: User
     database_id: Optional[int]
     enable_embedding: bool
@@ -37,12 +37,12 @@ class Card(GenericTemplateWithArchive):
     creator_id: int
     updated_at: datetime
     made_public_by_id: Optional[int]
-    embedding_params: Optional[dict]
+    embedding_params: Optional[dict[str, Any]]
     cache_ttl: Optional[str]
-    dataset_query: dict
+    dataset_query: dict[str, Any]
     display: str
-    last_edit_info: Optional[dict] = Field(alias="last-edit-info")
-    visualization_settings: dict
+    last_edit_info: Optional[dict[str, Any]] = Field(alias="last-edit-info")
+    visualization_settings: dict[str, Any]
     collection: Optional[Collection]
     dataset: Optional[int]
     created_at: datetime
@@ -52,89 +52,9 @@ class Card(GenericTemplateWithArchive):
     is_favorite: Optional[bool] = Field(alias="favorite")
 
     @classmethod
-    def get(
-        cls, adapter: MetabaseApi, targets: Optional[list[int]] = None
-    ) -> list[Self]:
-        """Fetch a list of cards using the provided MetabaseAPI
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            targets (list[int], optional): Card IDs to fetch or returns all
-
-        Returns:
-            list[Card]: List of cards requested
-        """
-        return super(Card, cls).get(adapter=adapter, targets=targets)
-
-    @classmethod
-    def create(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
-        """Create new cards
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            payloads (list[dict]): Details of cards to create
-
-        Returns:
-            list[Card]: List of cards created
-        """
-        return super(Card, cls).create(adapter=adapter, payloads=payloads)
-
-    @classmethod
-    def update(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
-        """Update existing cards
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            payloads (list[dict]): Details of cards to update
-
-        Returns:
-            list[Card]: List of updated cards
-        """
-        return super(Card, cls).update(adapter=adapter, payloads=payloads)
-
-    @classmethod
-    def archive(
-        cls, adapter: MetabaseApi, targets: list[int], unarchive=False
-    ) -> list[Self]:
-        """Archive cards
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            targets (list[int]): Card IDs to archive
-            unarchive (bool, optional): Unarchive targets. Defaults to False.
-
-        Returns:
-            list[Card]: List of archived cards
-        """
-        return super(Card, cls).archive(
-            adapter=adapter, targets=targets, unarchive=unarchive
-        )
-
-    @classmethod
-    def search(
-        cls,
-        adapter: MetabaseApi,
-        search_params: list[dict],
-        search_list: Optional[list] = None,
-    ) -> list[Self]:
-        """Search for cards based on provided criteria
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            search_params (list[dict]): Search criteria; 1 result per
-            search_list (list, optional): Search existing list or pulls from API
-
-        Returns:
-            list[Card]: List of cards from results
-        """
-        return super(Card, cls).search(
-            adapter=adapter,
-            search_params=search_params,
-            search_list=search_list,
-        )
-
-    @classmethod
-    def related(cls, adapter: MetabaseApi, targets: list[int]) -> list[dict]:
+    def related(
+        cls: type[GA], adapter: MetabaseApi, targets: list[int]
+    ) -> list[dict[str, Any]]:
         """Objects related to targets
 
         Args:
@@ -154,7 +74,7 @@ class Card(GenericTemplateWithArchive):
         return results
 
     @classmethod
-    def embeddable(cls, adapter: MetabaseApi) -> list[Self]:
+    def embeddable(cls: type[GA], adapter: MetabaseApi) -> list[GA]:
         """Fetch list of cards with embedding enabled
 
         Args:
@@ -172,7 +92,9 @@ class Card(GenericTemplateWithArchive):
         raise EmptyDataReceived
 
     @classmethod
-    def favorite(cls, adapter: MetabaseApi, targets: list[int]) -> list[dict]:
+    def favorite(
+        cls: type[GA], adapter: MetabaseApi, targets: list[int]
+    ) -> list[dict[str, Any]]:
         """Favorite cards
 
         Args:
@@ -196,7 +118,9 @@ class Card(GenericTemplateWithArchive):
         return results
 
     @classmethod
-    def unfavorite(cls, adapter: MetabaseApi, targets: list[int]) -> list[dict]:
+    def unfavorite(
+        cls: type[GA], adapter: MetabaseApi, targets: list[int]
+    ) -> list[dict[str, Any]]:
         """Unfavorite cards
 
         Args:
@@ -208,6 +132,7 @@ class Card(GenericTemplateWithArchive):
         """
         results = []
         for target in targets:
+            result: dict[str, int | bool | str] = {}
             try:
                 _ = adapter.delete(endpoint=f"/card/{target}/favorite")
                 result = {"card_id": target, "success": True}
@@ -220,7 +145,9 @@ class Card(GenericTemplateWithArchive):
         return results
 
     @classmethod
-    def share(cls, adapter: MetabaseApi, targets: list[int]) -> list[dict]:
+    def share(
+        cls: type[GA], adapter: MetabaseApi, targets: list[int]
+    ) -> list[dict[str, Any]]:
         """Generate publicly-accessible links for cards
 
         Args:
@@ -238,7 +165,9 @@ class Card(GenericTemplateWithArchive):
         )
 
     @classmethod
-    def unshare(cls, adapter: MetabaseApi, targets: list[int]) -> list[dict]:
+    def unshare(
+        cls: type[GA], adapter: MetabaseApi, targets: list[int]
+    ) -> list[dict[str, Any]]:
         """Remove publicly-accessible links for cards
 
         Args:
