@@ -1,21 +1,19 @@
 """Classes related to collections endpoints
 """
+from __future__ import annotations  # Included for support of |
 
-from typing import ClassVar, Optional
-
-from typing_extensions import Self
+from typing import Any, ClassVar, Optional
 
 from metabase_tools.exceptions import EmptyDataReceived
 from metabase_tools.metabase import MetabaseApi
-from metabase_tools.models.generic import GenericTemplateWithArchive
+from metabase_tools.models.generic import GA, GenericWithArchive
 
 
-class Collection(GenericTemplateWithArchive):
+class Collection(GenericWithArchive):
     """Collection object class with related methods"""
 
     BASE_EP: ClassVar[str] = "/collection"
 
-    id: int | str  # root is a valid id for a collection
     description: Optional[str]
     archived: Optional[bool]
     slug: Optional[str]
@@ -24,96 +22,12 @@ class Collection(GenericTemplateWithArchive):
     location: Optional[str]
     namespace: Optional[int]
     effective_location: Optional[str]
-    effective_ancestors: Optional[list[dict]]
+    effective_ancestors: Optional[list[dict[str, Any]]]
     can_write: Optional[bool]
     parent_id: Optional[int]
 
     @classmethod
-    def get(
-        cls, adapter: MetabaseApi, targets: Optional[list[int]] = None
-    ) -> list[Self]:
-        """Fetch a list of collections using the provided MetabaseAPI
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            targets (list[int], optional): Collection IDs to fetch or returns all
-
-        Returns:
-            list[Collection]: List of collections requested
-        """
-        return super(Collection, cls).get(adapter=adapter, targets=targets)
-
-    @classmethod
-    def create(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
-        """Create new collections
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            payloads (list[dict]): Details of collections to create
-
-        Returns:
-            list[Collection]: List of collections created
-        """
-        return super(Collection, cls).create(adapter=adapter, payloads=payloads)
-
-    @classmethod
-    def update(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
-        """Update existing collections
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            payloads (list[dict]): Details of collections to update
-
-        Returns:
-            list[Collection]: List of updated collections
-        """
-        return super(Collection, cls).update(adapter=adapter, payloads=payloads)
-
-    @classmethod
-    def archive(
-        cls, adapter: MetabaseApi, targets: list[int], unarchive=False
-    ) -> list[Self]:
-        """Archive collections
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            targets (list[int]): Collection IDs to archive
-            unarchive (bool, optional): Unarchive targets. Defaults to False.
-
-        Returns:
-            list[Collection]: List of archived collections
-        """
-        return super(Collection, cls).archive(
-            adapter=adapter,
-            targets=targets,
-            unarchive=unarchive,
-        )
-
-    @classmethod
-    def search(
-        cls,
-        adapter: MetabaseApi,
-        search_params: list[dict],
-        search_list: Optional[list] = None,
-    ) -> list[Self]:
-        """Search for collection based on provided criteria
-
-        Args:
-            adapter (MetabaseApi): Connection to Metabase API
-            search_params (list[dict]): Search criteria; 1 result per
-            search_list (list, optional): Search existing list or pulls from API
-
-        Returns:
-            list[Collection]: List of collections from results
-        """
-        return super(Collection, cls).search(
-            adapter=adapter,
-            search_params=search_params,
-            search_list=search_list,
-        )
-
-    @classmethod
-    def get_tree(cls, adapter: MetabaseApi) -> list[dict]:
+    def get_tree(cls: type[GA], adapter: MetabaseApi) -> list[dict[str, Any]]:
         """Collection tree
 
         Args:
@@ -133,7 +47,7 @@ class Collection(GenericTemplateWithArchive):
         raise EmptyDataReceived
 
     @staticmethod
-    def _flatten_tree(parent: dict, path: str = "/") -> list[dict]:
+    def _flatten_tree(parent: dict[str, Any], path: str = "/") -> list[dict[str, Any]]:
         """Recursive function to flatten collection tree to show the full path for all\
              collections
 
@@ -166,7 +80,7 @@ class Collection(GenericTemplateWithArchive):
         return children
 
     @classmethod
-    def get_flat_list(cls, adapter: MetabaseApi) -> list[dict]:
+    def get_flat_list(cls, adapter: MetabaseApi) -> list[dict[str, Any]]:
         """Flattens collection tree so the full path of each collection is shown
 
         Args:
@@ -197,7 +111,7 @@ class Collection(GenericTemplateWithArchive):
         collection_id: int,
         model_type: Optional[str] = None,
         archived: bool = False,
-    ) -> list:
+    ) -> list[dict[str, Any]]:
         """Get the contents of the provided collection
 
         Args:
@@ -212,7 +126,7 @@ class Collection(GenericTemplateWithArchive):
         Returns:
             list: Contents of collection
         """
-        params = {}
+        params: dict[Any, Any] = {}
         if archived:
             params["archived"] = archived
         if model_type:
@@ -230,7 +144,7 @@ class Collection(GenericTemplateWithArchive):
         raise EmptyDataReceived
 
     @classmethod
-    def graph(cls, adapter: MetabaseApi) -> dict:
+    def graph(cls, adapter: MetabaseApi) -> dict[str, Any]:
         """Graph of collection
 
         Args:
