@@ -1,11 +1,11 @@
 """
     Generic classes for Metabase
 """
+from __future__ import annotations  # Included for support of |
 
-from typing import ClassVar, Optional
+from typing import Any, ClassVar, Optional, TypeVar
 
 from pydantic import BaseModel
-from typing_extensions import Self
 
 from metabase_tools.exceptions import EmptyDataReceived, InvalidParameters
 from metabase_tools.metabase import MetabaseApi
@@ -16,7 +16,7 @@ class MetabaseGenericObject(BaseModel, extra="forbid"):
 
     BASE_EP: ClassVar[str]
 
-    id: int
+    id: int | str
     name: str
 
     @classmethod
@@ -25,8 +25,8 @@ class MetabaseGenericObject(BaseModel, extra="forbid"):
         http_method: str,
         adapter: MetabaseApi,
         endpoint: str,
-        source: list[int] | list[dict],
-    ) -> list[dict]:
+        source: list[int] | list[dict[str, Any]],
+    ) -> list[dict[str, Any]]:
         """Sends requests to API based on a list of objects
 
         Args:
@@ -52,8 +52,7 @@ class MetabaseGenericObject(BaseModel, extra="forbid"):
             elif isinstance(item, dict):
                 item_ep = endpoint.format(**item)
                 if http_method == "PUT":
-                    response = adapter.generic_request(
-                        http_method=http_method,
+                    response = adapter.put(
                         endpoint=item_ep,
                         json=item,
                     )
@@ -74,13 +73,13 @@ class MetabaseGenericObject(BaseModel, extra="forbid"):
         raise EmptyDataReceived("No data returned")
 
 
-class GenericGet(MetabaseGenericObject):
-    """Generic class for objects with get support in the API"""
+class GenericWithoutArchive(MetabaseGenericObject):
+    """Generic class for objects without support for archive"""
 
     @classmethod
     def get(
-        cls, adapter: MetabaseApi, targets: Optional[list[int]] = None
-    ) -> list[Self]:
+        cls: type[GWA], adapter: MetabaseApi, targets: Optional[list[int]] = None
+    ) -> list[GWA]:
         """Generic method for returning an object or list of objects
 
         Args:
@@ -119,11 +118,11 @@ class GenericGet(MetabaseGenericObject):
 
     @classmethod
     def search(
-        cls,
+        cls: type[GWA],
         adapter: MetabaseApi,
-        search_params: list[dict],
-        search_list: Optional[list[Self]] = None,
-    ) -> list[Self]:
+        search_params: list[dict[str, Any]],
+        search_list: Optional[list[GWA]] = None,
+    ) -> list[GWA]:
         """Method to search a list of objects meeting a list of parameters
 
         Args:
@@ -136,7 +135,7 @@ class GenericGet(MetabaseGenericObject):
         Returns:
             list[Self]: List of objects of the relevant type
         """
-        objs = search_list or cls.get(adapter=adapter)  # type: ignore
+        objs = search_list or cls.get(adapter=adapter)
         results = []
         for param in search_params:
             for obj in objs:
@@ -146,12 +145,10 @@ class GenericGet(MetabaseGenericObject):
                         results.append(obj)
         return results
 
-
-class GenericCreate(MetabaseGenericObject):
-    """Generic class for objects with create support in the API"""
-
     @classmethod
-    def create(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
+    def create(
+        cls: type[GWA], adapter: MetabaseApi, payloads: list[dict[str, Any]]
+    ) -> list[GWA]:
         """Generic method for creating a list of objects
 
         Args:
@@ -176,12 +173,10 @@ class GenericCreate(MetabaseGenericObject):
         # If something other than dict or list[dict], raise error
         raise InvalidParameters("Invalid target(s)")
 
-
-class GenericUpdate(MetabaseGenericObject):
-    """Generic class for objects with update support in the API"""
-
     @classmethod
-    def update(cls, adapter: MetabaseApi, payloads: list[dict]) -> list[Self]:
+    def update(
+        cls: type[GWA], adapter: MetabaseApi, payloads: list[dict[str, Any]]
+    ) -> list[GWA]:
         """Generic method for updating a list of objects
 
         Args:
@@ -206,12 +201,8 @@ class GenericUpdate(MetabaseGenericObject):
         # If something other than dict or list[dict], raise error
         raise InvalidParameters("Invalid target(s)")
 
-
-class GenericDelete(MetabaseGenericObject):
-    """Generic class for objects with delete support in the API"""
-
     @classmethod
-    def delete(cls, adapter: MetabaseApi, targets: list[int]) -> dict:
+    def delete(cls, adapter: MetabaseApi, targets: list[int]) -> dict[int, Any]:
         """Method to delete a list of objects
 
         Args:
@@ -235,16 +226,16 @@ class GenericDelete(MetabaseGenericObject):
         raise InvalidParameters("Invalid set of targets")
 
 
-class GenericArchive(MetabaseGenericObject):
+class GenericWithArchive(GenericWithoutArchive):
     """Generic class for objects with archive support in the API"""
 
     @classmethod
     def archive(
-        cls,
+        cls: type[GA],
         adapter: MetabaseApi,
         targets: list[int],
-        unarchive: bool,
-    ) -> list[Self]:
+        unarchive: bool = False,
+    ) -> list[GA]:
         """Generic method for archiving a list of objects
 
         Args:
@@ -271,13 +262,6 @@ class GenericArchive(MetabaseGenericObject):
         raise InvalidParameters("Invalid set of targets")
 
 
-class GenericTemplateWithArchive(
-    GenericGet, GenericCreate, GenericUpdate, GenericDelete, GenericArchive
-):
-    pass
-
-
-class GenericTemplateWithoutArchive(
-    GenericGet, GenericCreate, GenericUpdate, GenericDelete
-):
-    pass
+MGO = TypeVar("MGO", bound="MetabaseGenericObject")
+GWA = TypeVar("GWA", bound="GenericWithoutArchive")
+GA = TypeVar("GA", bound="GenericWithArchive")
