@@ -1,16 +1,16 @@
 """
 MetabaseTools extends MetabaseApi with additional complex functions
 """
+from __future__ import annotations  # Included for support of |
+
 from json import dumps, loads
 from pathlib import Path
-from typing import Optional
+from typing import Any, Optional
 
 from metabase_tools.exceptions import (
     EmptyDataReceived,
-    InvalidParameters,
     ItemInPersonalCollection,
     ItemNotFound,
-    NoUpdateProvided,
 )
 from metabase_tools.metabase import MetabaseApi
 from metabase_tools.models.card import Card
@@ -63,11 +63,7 @@ class MetabaseTools(MetabaseApi):
         )
 
         # Format filtered list
-        formatted_list = {
-            "root": str(root_folder),
-            "file_extension": file_extension,
-            "cards": [],
-        }
+        cards_to_write = []
 
         for card in cards:
             try:
@@ -77,7 +73,7 @@ class MetabaseTools(MetabaseApi):
             except ItemInPersonalCollection:
                 self._logger.warning("Skipping %s (personal collection)", card.name)
                 continue
-            formatted_list["cards"].append(new_card)
+            cards_to_write.append(new_card)
 
             try:
                 self._save_query(
@@ -89,7 +85,9 @@ class MetabaseTools(MetabaseApi):
                 self._logger.warning("Skipping %s (name error)", card.name)
                 continue
             self._logger.debug(
-                "%s saved to %s", card.name, f"{root_folder}/{new_card['path']}"
+                "%s saved to %s",
+                card.name,
+                f"{root_folder}/{new_card['path']}",
             )
 
         # Save mapping file
@@ -100,7 +98,7 @@ class MetabaseTools(MetabaseApi):
             "Completed iterating through list, saving file: %s", mapping_path
         )
         with open(mapping_path, "w", newline="", encoding="utf-8") as file:
-            file.write(dumps(formatted_list, indent=2))
+            file.write(dumps(cards_to_write, indent=2))
 
         # Returns path to file saved
         return mapping_path
@@ -108,9 +106,10 @@ class MetabaseTools(MetabaseApi):
     def upload_native_queries(
         self,
         mapping_path: Path | str,
+        file_extension: str,
         dry_run: bool = True,
         stop_on_error: bool = False,
-    ) -> list[dict] | dict:
+    ) -> list[dict[str, Any]] | dict[str, Any]:
         """Uploads queries to Metabase
 
         Args:
@@ -127,39 +126,23 @@ class MetabaseTools(MetabaseApi):
         Returns:
             list[dict] | dict: Results of upload
         """
-        # Determine mapping path
-        mapping_path = Path(mapping_path or "./mapping.json")
-
         # Open mapping configuration file
+        mapping_path = Path(mapping_path or "./mapping.json")
         with open(mapping_path, "r", newline="", encoding="utf-8") as file:
-            mapping = loads(file.read())
-
-        # Initialize common settings (e.g. root folder, file extension, etc.)
-        root_folder = Path(mapping.get("root", "."))
-        extension = mapping.get("file_extension", ".sql")
-        cards = mapping["cards"]
+            cards = loads(file.read())
 
         # Iterate through mapping file
-        changes = {"updates": [], "creates": [], "errors": []}
-        collections_by_id = self._get_collections_dict(key="id")
+        changes: dict[str, list[dict[str, Any]]] = {
+            "updates": [],
+            "creates": [],
+            "errors": [],
+        }
         collections_by_path = self._get_collections_dict(key="path")
         for card in cards:
-            card_path = Path(f"{root_folder}/{card['path']}/{card['name']}.{extension}")
-            if (
-                card_path.exists() and "id" in card
-            ):  # Ensures file exists and id is present
-                try:
-                    update = self._update_existing_card(
-                        dev_card=card,
-                        card_path=card_path,
-                        collections_by_id=collections_by_id,
-                        collections_by_path=collections_by_path,
-                    )
-                except NoUpdateProvided:
-                    self._logger.debug("No updates necessary for %s", card["name"])
-                    continue
-                changes["updates"].append(update)
-            elif card_path.exists():
+            card_path = Path(
+                f"{mapping_path.parent}/{card['path']}/{card['name']}.{file_extension}"
+            )
+            if card_path.exists():
                 # Check if a card with the same name exists in the listed location
                 dev_coll_id = collections_by_path[card["path"]]["id"]
                 try:
@@ -175,37 +158,20 @@ class MetabaseTools(MetabaseApi):
                     )
                     card_id = None
 
-                if card_id:  # update card
-                    prod_card = Card.get(adapter=self, targets=[card_id])[0]
-                    with open(card_path, "r", newline="", encoding="utf-8") as file:
-                        dev_code = file.read()
-                    if dev_code != prod_card.dataset_query["native"]["query"]:
-                        dev_query = prod_card.dataset_query.copy()
-                        dev_query["native"]["query"] = dev_code
-                        dev_def = {"id": card_id, "dataset_query": dev_query}
-                        changes["updates"].append(dev_def)
+                if card_id:  # update existing card
+                    card_result = self._update_existing_card(
+                        card_id=card_id, card_path=card_path
+                    )
+                    if card_result:
+                        changes["updates"].append(card_result)
                 else:  # create card
-                    with open(card_path, "r", newline="", encoding="utf-8") as file:
-                        dev_query = file.read()
-                    db_id = [
-                        database.id
-                        for database in Database.get(adapter=self)
-                        if card["database"] == database.name
-                    ][0]
+                    card_result = self._create_new_card(
+                        card=card, card_path=card_path, dev_coll_id=dev_coll_id
+                    )
+                    if card_result:
+                        changes["creates"].append(card_result)
 
-                    new_card_def = {
-                        "visualization_settings": {},
-                        "collection_id": dev_coll_id,
-                        "name": card["name"],
-                        "dataset_query": {
-                            "type": "native",
-                            "native": {"query": dev_query},
-                            "database": db_id,
-                        },
-                        "display": "table",
-                    }
-                    changes["creates"].append(new_card_def.copy())
-            else:
+            else:  # File does not exist
                 self._logger.error("Skipping %s (file not found)", card["name"])
                 if stop_on_error:
                     raise FileNotFoundError(f"{card_path} not found")
@@ -214,31 +180,38 @@ class MetabaseTools(MetabaseApi):
         # Loop exit before pushing changes to Metabase in case errors are encountered
         # Push changes back to Metabase API
         if not dry_run:
-            results = []
-            if len(changes["updates"]) > 0:
-                update_results = Card.update(adapter=self, payloads=changes["updates"])
-                if isinstance(update_results, list):
-                    for result in update_results:
-                        results.append(
-                            {"id": result.id, "name": result.name, "is_success": True}
-                        )
-
-            if len(changes["creates"]) > 0:
-                create_results = Card.create(adapter=self, payloads=changes["creates"])
-                if isinstance(create_results, list):
-                    for result in create_results:
-                        results.append(
-                            {"id": result.id, "name": result.name, "is_success": True}
-                        )
-
-            return results
+            return self._execute_changes(changes)
         return changes
 
-    def _get_mapping_details(self, card: Card, collections_by_id: dict) -> dict:
+    def _execute_changes(
+        self, changes: dict[str, list[dict[str, Any]]]
+    ) -> list[dict[str, Any]] | dict[str, Any]:
+        results = []
+        if len(changes["updates"]) > 0:
+            update_results = Card.update(adapter=self, payloads=changes["updates"])
+            if isinstance(update_results, list):
+                for result in update_results:
+                    results.append(
+                        {"id": result.id, "name": result.name, "is_success": True}
+                    )
+
+        if len(changes["creates"]) > 0:
+            create_results = Card.create(adapter=self, payloads=changes["creates"])
+            if isinstance(create_results, list):
+                for result in create_results:
+                    results.append(
+                        {"id": result.id, "name": result.name, "is_success": True}
+                    )
+
+        return results
+
+    def _get_mapping_details(
+        self, card: Card, collections_by_id: dict[Any, Any]
+    ) -> dict[str, Any]:
         try:
             mapping_details = {
-                "id": card.id,
                 "name": card.name,
+                "collection_id": card.collection_id,
                 "path": collections_by_id[card.collection_id]["path"],
             }
         except KeyError as error_raised:
@@ -246,13 +219,14 @@ class MetabaseTools(MetabaseApi):
                 "Item in personal collection"
             ) from error_raised
 
-        mapping_details["database"] = Database.search(
+        mapping_details["database_id"] = card.database_id
+        mapping_details["database_name"] = Database.search(
             adapter=self, search_params=[{"id": card.database_id}]
         )[0].name
 
         return mapping_details
 
-    def _save_query(self, card: Card, save_path: str, file_extension):
+    def _save_query(self, card: Card, save_path: str, file_extension: str) -> None:
         # SQL file creation
         sql_code = card.dataset_query["native"]["query"]
         sql_path = Path(f"{save_path}")
@@ -261,55 +235,55 @@ class MetabaseTools(MetabaseApi):
         with open(sql_path, "w", newline="", encoding="utf-8") as file:
             file.write(sql_code)
 
-    def _get_collections_dict(self, key: str):
+    def _get_collections_dict(self, key: str) -> dict[Any, Any]:
         collections = Collection.get_flat_list(adapter=self)
         non_keys = [k for k in collections[0].keys() if k != key]
         return {item[key]: {nk: item[nk] for nk in non_keys} for item in collections}
 
-    def _translate_path(
-        self, collection_id: Optional[int] = None, collection_path: Optional[str] = None
-    ) -> int | str:
-        if collection_id:
-            collections_by_id = self._get_collections_dict(key="id")
-            return collections_by_id[collection_id]["path"]
-
-        if collection_path:
-            collections_by_path = self._get_collections_dict(key="path")
-            return collections_by_path[collection_path]["id"]
-
-        raise InvalidParameters
-
     def _find_card_id(self, card_name: str, collection_id: int) -> int:
         collection_items = Collection.get_contents(
             adapter=self, collection_id=collection_id, model_type="card", archived=False
         )
         for item in collection_items:
-            if item["name"] == card_name:
+            if item["name"] == card_name and isinstance(item["id"], int):
                 return item["id"]
         raise ItemNotFound
 
     def _update_existing_card(
         self,
-        dev_card: dict,
+        card_id: int,
         card_path: Path | str,
-        collections_by_id: dict,
-        collections_by_path: dict,
-    ) -> dict:
-        prod_card = Card.get(adapter=self, targets=[dev_card["id"]])[0]
-        # Get query definition
+    ) -> dict[str, Any]:
+        prod_card = Card.get(adapter=self, targets=[card_id])[0]
         with open(card_path, "r", newline="", encoding="utf-8") as file:
-            dev_card["query"] = file.read()
-        # Generate update
-        if (
-            dev_card["query"] != prod_card.dataset_query["native"]["query"]
-            or dev_card["path"] != collections_by_id[prod_card.collection_id]["path"]
-        ):
-            dev_card["dataset_query"] = prod_card.dataset_query.copy()
-            dev_card["dataset_query"]["native"]["query"] = dev_card["query"]
-            dev_card["collection_id"] = collections_by_path[dev_card["path"]]["id"]
-            return {
-                "id": dev_card["id"],
-                "dataset_query": dev_card["dataset_query"],
-                "collection_id": dev_card["collection_id"],
-            }
-        raise NoUpdateProvided
+            dev_code = file.read()
+        if dev_code != prod_card.dataset_query["native"]["query"]:
+            dev_query = prod_card.dataset_query.copy()
+            dev_query["native"]["query"] = dev_code
+            dev_def = {"id": card_id, "dataset_query": dev_query}
+            return dev_def
+        return {}
+
+    def _create_new_card(
+        self, card: dict[str, Any], card_path: Path, dev_coll_id: int
+    ) -> dict[str, Any]:
+        with open(card_path, "r", newline="", encoding="utf-8") as file:
+            dev_query = file.read()
+        db_id = [
+            database.id
+            for database in Database.get(adapter=self)
+            if card["database_name"] == database.name
+        ][0]
+
+        new_card_def = {
+            "visualization_settings": {},
+            "collection_id": dev_coll_id,
+            "name": card["name"],
+            "dataset_query": {
+                "type": "native",
+                "native": {"query": dev_query},
+                "database": db_id,
+            },
+            "display": "table",
+        }
+        return new_card_def
